/****************************************************************************
** Meta object code from reading C++ file 'qtasktree.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qttasktree/src/tasktree/qtasktree.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qtasktree.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10QtTaskTree14QTaskInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto QtTaskTree::QTaskInterface::qt_create_metaobjectdata<qt_meta_tag_ZN10QtTaskTree14QTaskInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtTaskTree::QTaskInterface",
        "done",
        "",
        "QtTaskTree::DoneResult",
        "result"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'done'
        QtMocHelpers::SignalData<void(QtTaskTree::DoneResult)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QTaskInterface, qt_meta_tag_ZN10QtTaskTree14QTaskInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QtTaskTree::QTaskInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QtTaskTree14QTaskInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QtTaskTree14QTaskInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10QtTaskTree14QTaskInterfaceE_t>.metaTypes,
    nullptr
} };

void QtTaskTree::QTaskInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QTaskInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->done((*reinterpret_cast<std::add_pointer_t<QtTaskTree::DoneResult>>(_a[1])), QPrivateSignal()); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QTaskInterface::*)(QtTaskTree::DoneResult , QPrivateSignal)>(_a, &QTaskInterface::done, 0))
            return;
    }
}

const QMetaObject *QtTaskTree::QTaskInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtTaskTree::QTaskInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QtTaskTree14QTaskInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QtTaskTree::QTaskInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void QtTaskTree::QTaskInterface::done(QtTaskTree::DoneResult _t1, QPrivateSignal _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN10QtTaskTree9QTaskTreeE_t {};
} // unnamed namespace

template <> constexpr inline auto QtTaskTree::QTaskTree::qt_create_metaobjectdata<qt_meta_tag_ZN10QtTaskTree9QTaskTreeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtTaskTree::QTaskTree",
        "started",
        "",
        "done",
        "QtTaskTree::DoneWith",
        "result",
        "asyncCountChanged",
        "qsizetype",
        "count",
        "progressValueChanged",
        "value"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'started'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'done'
        QtMocHelpers::SignalData<void(QtTaskTree::DoneWith)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'asyncCountChanged'
        QtMocHelpers::SignalData<void(qsizetype)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'progressValueChanged'
        QtMocHelpers::SignalData<void(qsizetype)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QTaskTree, qt_meta_tag_ZN10QtTaskTree9QTaskTreeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QtTaskTree::QTaskTree::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QtTaskTree9QTaskTreeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QtTaskTree9QTaskTreeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10QtTaskTree9QTaskTreeE_t>.metaTypes,
    nullptr
} };

void QtTaskTree::QTaskTree::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QTaskTree *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->started(); break;
        case 1: _t->done((*reinterpret_cast<std::add_pointer_t<QtTaskTree::DoneWith>>(_a[1]))); break;
        case 2: _t->asyncCountChanged((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1]))); break;
        case 3: _t->progressValueChanged((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QTaskTree::*)()>(_a, &QTaskTree::started, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QTaskTree::*)(QtTaskTree::DoneWith )>(_a, &QTaskTree::done, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QTaskTree::*)(qsizetype )>(_a, &QTaskTree::asyncCountChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QTaskTree::*)(qsizetype )>(_a, &QTaskTree::progressValueChanged, 3))
            return;
    }
}

const QMetaObject *QtTaskTree::QTaskTree::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtTaskTree::QTaskTree::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QtTaskTree9QTaskTreeE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QtTaskTree::QTaskTree::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void QtTaskTree::QTaskTree::started()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QtTaskTree::QTaskTree::done(QtTaskTree::DoneWith _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QtTaskTree::QTaskTree::asyncCountChanged(qsizetype _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QtTaskTree::QTaskTree::progressValueChanged(qsizetype _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN10QtTaskTreeE_t {};
} // unnamed namespace

template <> constexpr inline auto QtTaskTree::qt_create_metaobjectdata<qt_meta_tag_ZN10QtTaskTreeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtTaskTree",
        "WorkflowPolicy",
        "StopOnError",
        "ContinueOnError",
        "StopOnSuccess",
        "ContinueOnSuccess",
        "StopOnSuccessOrError",
        "FinishAllAndSuccess",
        "FinishAllAndError",
        "SetupResult",
        "Continue",
        "StopWithSuccess",
        "StopWithError",
        "DoneResult",
        "Success",
        "Error",
        "DoneWith",
        "Cancel",
        "CallDone",
        "CallDoneFlag",
        "Never",
        "OnSuccess",
        "OnError",
        "OnCancel",
        "Always"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'WorkflowPolicy'
        QtMocHelpers::EnumData<WorkflowPolicy>(1, 1, QMC::EnumIsScoped).add({
            {    2, WorkflowPolicy::StopOnError },
            {    3, WorkflowPolicy::ContinueOnError },
            {    4, WorkflowPolicy::StopOnSuccess },
            {    5, WorkflowPolicy::ContinueOnSuccess },
            {    6, WorkflowPolicy::StopOnSuccessOrError },
            {    7, WorkflowPolicy::FinishAllAndSuccess },
            {    8, WorkflowPolicy::FinishAllAndError },
        }),
        // enum 'SetupResult'
        QtMocHelpers::EnumData<SetupResult>(9, 9, QMC::EnumIsScoped).add({
            {   10, SetupResult::Continue },
            {   11, SetupResult::StopWithSuccess },
            {   12, SetupResult::StopWithError },
        }),
        // enum 'DoneResult'
        QtMocHelpers::EnumData<DoneResult>(13, 13, QMC::EnumIsScoped).add({
            {   14, DoneResult::Success },
            {   15, DoneResult::Error },
        }),
        // enum 'DoneWith'
        QtMocHelpers::EnumData<DoneWith>(16, 16, QMC::EnumIsScoped).add({
            {   14, DoneWith::Success },
            {   15, DoneWith::Error },
            {   17, DoneWith::Cancel },
        }),
        // flag 'CallDone'
        QtMocHelpers::EnumData<CallDone>(18, 19, QMC::EnumIsFlag | QMC::EnumIsScoped).add({
            {   20, CallDoneFlag::Never },
            {   21, CallDoneFlag::OnSuccess },
            {   22, CallDoneFlag::OnError },
            {   23, CallDoneFlag::OnCancel },
            {   24, CallDoneFlag::Always },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10QtTaskTreeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10QtTaskTreeE =
    QtTaskTree::qt_create_metaobjectdata<qt_meta_tag_ZN10QtTaskTreeE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10QtTaskTreeE =
    qt_staticMetaObjectContent_ZN10QtTaskTreeE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10QtTaskTreeE =
    qt_staticMetaObjectContent_ZN10QtTaskTreeE.relocatingData;

Q_CONSTINIT const QMetaObject QtTaskTree::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10QtTaskTreeE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10QtTaskTreeE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10QtTaskTreeE.metaTypes,
    nullptr
} };

QT_WARNING_POP
